#ifndef INIT_H
#define INIT_H

#include "at8xc51snd1.h"
#include <intrins.h>

char spi_data_in = 0x00;
char spi_data_out = 0x00;
char uart_in_data = 0;
char uart_out_data = 0;

bit spi_transmit = 0;
bit SRC = 0x9b;
bit uart_in = 0;
bit uart_out = 0;
bit t0_event = 0;

int t0_count = 0;

#define	QUARZ	20000		/* Frequenz in kHz */
#define Q_Low	LOW(QUARZ)	/* Untere 8 Bit Quarzfrequenz */
#define Q_High	HIGH(QUARZ) /* Obere 8 Bit Quarzfrequenz */

#define LED1	P5_1

int T0_TIME = (65534 - 3300); /* Alle 1ms Interrupt */

void init()
{
	CKCON |= 0x01;				/* X2 Modus */

	SPCON |= 0x10;              /* Master mode */
	SPCON |= 0x01;				/* Fclk Periph/4 -> 5MHz */
	SPCON |= 0x20;              /* SS disable */
	SPCON |= 0x08;              /* CPOL=1; */
	SPCON |= 0x04;              /* CPHA=1; */
	SPCON |= 0x40;              /* run spi */
	
	/* Timer0 u. Timer1 Setup */
	TMOD &= 0x00;				// Timer 0 & 1 mode 1 with software gate 
	TMOD |= 0x11;				// Timer 0 & 1 als Zeitgeber einrichten	
	TL0 = T0_TIME;				// Timer0 1ms
	TH0 = T0_TIME>>8;									            

	/* Lsche Interrupt-Flags */
	TF0 = 0;					// Lsche Timer0-Interrupt-Flag
	TF1 = 0;					// Lsche Timer1-Interrupt-Flag

	/* UART Setup */
	/*ES = 1;						// enable UART interrupt
	SM0 = 0;					// Mode 1: 8 Bit UART, variable Baudrate
	SM1 = 1;					// Mode 1: 8 Bit UART, variable Baudrate
	SM2 = 0;					// Multiprozessor-Kommunikation deaktivieren
	BRL = 178;					// Baud Rate Reload Wert
	BDRCON = BDRCON|0x1e;		// BDRCON Register: BRR = 1, TBCK = 1, RBCK = 1, SPD = 1, SRC = 0
	SRC = 0;					// Baud Rate Source Select Bit
	REN = 1;					// Receiver Enable (UART Port)*/

	/* Interrupts freigeben */
	IEN1  |= 0x04;				/* enable spi interrupt */
	ET0 = 1;                	/* enable timer0 interrupt */
	EA  = 1;					// enable global interrupts

	TR0 = 0;					// disable/enable timer0		    	
}

void spi_routine(void) interrupt 9 		
{
	switch	( SPSTA )         	/* read and clear spi status register */
		{
		case 0x80:
		SPSTA &= 0x00;			/* set Bit 7 auf 0 (Data transfer completed = 0*/
		spi_transmit = 1;		/* set software flag */	
		spi_data_in = SPDAT;   	/* read receive data */
      	break;

		case 0x10:
		SPSTA &= 0x00;			/* set Bit 7 auf 0 (Data transfer completed = 0*/
       	spi_transmit = 1;		/* set software flag */
		spi_data_in = SPDAT;   	/* read receive data */
       	break;

		case 0x40:
		SPSTA &= 0x00;			/* set Bit 7 auf 0 (Data transfer completed = 0*/
       	spi_transmit = 1;		/* set software flag */
		spi_data_in = SPDAT;   	/* read receive data */
       	break;
		}
}

void serial_isr(void) interrupt 4
{
	if(TI)						/* Daten empfangen */
	{
		TI = 0;	  				/* Sende-Flag zurcksetzen */
		uart_out = 1;			/* "Gesendet" markieren */
	} 
	else if(RI)					/* Daten gesendet */
	{
		RI = 0;	  				/* Empfangs-Flag zurcksetzen */
		uart_in = 1;			/* "Empfangen" markieren */
		uart_in_data = SBUF;	/* Daten einlesen */
	}	
}

void timer0(void) interrupt 1 	/* Timer0 Interrupt Routine */
{
	TR0 = 0;
	TL0 = T0_TIME;
	TH0 = T0_TIME>>8;
	TF0 = 0;					/* Timer0-Flag zurcksetzen */
	TR0 = 1;
	t0_event = 1;
	t0_count++;
}

void delay_ms(int time)
{
	TR0 = 1;					/* run Timer0 */
	for(;time > 0; time--)
	{
		while(!t0_event);
		t0_event = 0;
	}
}

#endif